@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Parity Technologies (UK) Ltd.
+// Copyright 2019-2022 Parity Technologies (UK) Ltd.
 // This file is part of subxt.
 //
 // subxt is free software: you can redistribute it and/or modify
@@ -14,21 +14,45 @@
 // You should have received a copy of the GNU General Public License
 // along with subxt.  If not, see <http://www.gnu.org/licenses/>.
 
+//! Generate code for submitting extrinsics and query storage of a Substrate runtime.
+//!
+//! ## Note
+//!
+//! By default the codegen will search for the `System` pallet's `Account` storage item, which is
+//! the conventional location where an account's index (aka nonce) is stored.
+//!
+//! If this `System::Account` storage item is discovered, then it is assumed that:
+//!
+//!   1. The type of the storage item is a `struct` (aka a composite type)
+//!   2. There exists a field called `nonce` which contains the account index.
+//!
+//! These assumptions are based on the fact that the `frame_system::AccountInfo` type is the default
+//! configured type, and that the vast majority of chain configurations will use this.
+//!
+//! If either of these conditions are not satisfied, the codegen will fail.
+
 mod calls;
+mod constants;
+mod errors;
 mod events;
 mod storage;
 
 use super::GeneratedTypeDerives;
 use crate::{
     ir,
-    struct_def::StructDef,
-    types::TypeGenerator,
+    types::{
+        CompositeDef,
+        CompositeDefFields,
+        TypeGenerator,
+    },
 };
 use codec::Decode;
 use frame_metadata::{
     v14::RuntimeMetadataV14,
+    PalletMetadata,
     RuntimeMetadata,
     RuntimeMetadataPrefixed,
+    StorageEntryType,
 };
 use heck::SnakeCase as _;
 use proc_macro2::TokenStream as TokenStream2;
@@ -37,6 +61,7 @@ use quote::{
     format_ident,
     quote,
 };
+use scale_info::form::PortableForm;
 use std::{
     collections::HashMap,
     fs,
@@ -152,6 +177,7 @@ impl RuntimeGenerator {
                 )
             })
             .collect::<Vec<_>>();
+
         let modules = pallets_with_mod_names.iter().map(|(pallet, mod_name)| {
             let calls = if let Some(ref calls) = pallet.calls {
                 calls::generate_calls(&type_gen, pallet, calls, types_mod_ident)
@@ -171,12 +197,23 @@ impl RuntimeGenerator {
                 quote!()
             };
 
+            let constants_mod = if !pallet.constants.is_empty() {
+                constants::generate_constants(
+                    &type_gen,
+                    &pallet.constants,
+                    types_mod_ident,
+                )
+            } else {
+                quote!()
+            };
+
             quote! {
                 pub mod #mod_name {
                     use super::#types_mod_ident;
                     #calls
                     #event
                     #storage_mod
+                    #constants_mod
                 }
             }
         });
@@ -202,6 +239,12 @@ impl RuntimeGenerator {
         };
 
         let mod_ident = item_mod_ir.ident;
+        let pallets_with_constants =
+            pallets_with_mod_names
+                .iter()
+                .filter_map(|(pallet, pallet_mod_name)| {
+                    (!pallet.constants.is_empty()).then(|| pallet_mod_name)
+                });
         let pallets_with_storage =
             pallets_with_mod_names
                 .iter()
@@ -215,6 +258,20 @@ impl RuntimeGenerator {
                     pallet.calls.as_ref().map(|_| pallet_mod_name)
                 });
 
+        let error_details = errors::generate_error_details(&self.metadata);
+        let error_type = error_details.type_def;
+        let error_fn = error_details.dispatch_error_impl_fn;
+
+        let default_account_data_ident = format_ident!("DefaultAccountData");
+        let default_account_data_impl = generate_default_account_data_impl(
+            &pallets_with_mod_names,
+            &default_account_data_ident,
+            &type_gen,
+        );
+        let type_parameter_default_impl = default_account_data_impl
+            .as_ref()
+            .map(|_| quote!( = #default_account_data_ident ));
+
         quote! {
             #[allow(dead_code, unused_imports, non_camel_case_types)]
             pub mod #mod_ident {
@@ -222,76 +279,70 @@ impl RuntimeGenerator {
                 #( #modules )*
                 #types_mod
 
-                /// Default configuration of common types for a target Substrate runtime.
-                #[derive(Clone, Debug, Default, Eq, PartialEq)]
-                pub struct DefaultConfig;
+                /// The default error type returned when there is a runtime issue.
+                pub type DispatchError = self::runtime_types::sp_runtime::DispatchError;
 
-                impl ::subxt::Config for DefaultConfig {
-                    type Index = u32;
-                    type BlockNumber = u32;
-                    type Hash = ::subxt::sp_core::H256;
-                    type Hashing = ::subxt::sp_runtime::traits::BlakeTwo256;
-                    type AccountId = ::subxt::sp_runtime::AccountId32;
-                    type Address = ::subxt::sp_runtime::MultiAddress<Self::AccountId, u32>;
-                    type Header = ::subxt::sp_runtime::generic::Header<
-                        Self::BlockNumber, ::subxt::sp_runtime::traits::BlakeTwo256
-                    >;
-                    type Signature = ::subxt::sp_runtime::MultiSignature;
-                    type Extrinsic = ::subxt::sp_runtime::OpaqueExtrinsic;
+                // Statically generate error information so that we don't need runtime metadata for it.
+                #error_type
+                impl DispatchError {
+                    #error_fn
                 }
 
-                impl ::subxt::ExtrinsicExtraData<DefaultConfig> for DefaultConfig {
-                    type AccountData = AccountData;
-                    type Extra = ::subxt::DefaultExtra<DefaultConfig>;
-                }
+                #default_account_data_impl
 
-                pub type AccountData = self::system::storage::Account;
-
-                impl ::subxt::AccountData<DefaultConfig> for AccountData {
-                    fn nonce(result: &<Self as ::subxt::StorageEntry>::Value) -> <DefaultConfig as ::subxt::Config>::Index {
-                        result.nonce
-                    }
-                    fn storage_entry(account_id: <DefaultConfig as ::subxt::Config>::AccountId) -> Self {
-                        Self(account_id)
-                    }
-                }
-
-                pub struct RuntimeApi<T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>> {
+                pub struct RuntimeApi<T: ::subxt::Config, X, A #type_parameter_default_impl> {
                     pub client: ::subxt::Client<T>,
+                    marker: ::core::marker::PhantomData<(X, A)>,
                 }
 
-                impl<T> ::core::convert::From<::subxt::Client<T>> for RuntimeApi<T>
+                impl<T, X, A> ::core::convert::From<::subxt::Client<T>> for RuntimeApi<T, X, A>
                 where
-                    T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>,
+                    T: ::subxt::Config,
+                    X: ::subxt::SignedExtra<T>,
+                    A: ::subxt::AccountData,
                 {
                     fn from(client: ::subxt::Client<T>) -> Self {
-                        Self { client }
+                        Self { client, marker: ::core::marker::PhantomData }
                     }
                 }
 
-                impl<'a, T> RuntimeApi<T>
+                impl<'a, T, X, A> RuntimeApi<T, X, A>
                 where
-                    T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>,
+                    T: ::subxt::Config,
+                    X: ::subxt::SignedExtra<T>,
+                    A: ::subxt::AccountData,
                 {
+                    pub fn constants(&'a self) -> ConstantsApi {
+                        ConstantsApi
+                    }
+
                     pub fn storage(&'a self) -> StorageApi<'a, T> {
                         StorageApi { client: &self.client }
                     }
 
-                    pub fn tx(&'a self) -> TransactionApi<'a, T> {
-                        TransactionApi { client: &self.client }
+                    pub fn tx(&'a self) -> TransactionApi<'a, T, X, A> {
+                        TransactionApi { client: &self.client, marker: ::core::marker::PhantomData }
                     }
                 }
 
-                pub struct StorageApi<'a, T>
-                where
-                    T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>,
+                pub struct ConstantsApi;
+
+                impl ConstantsApi
                 {
+                    #(
+                        pub fn #pallets_with_constants(&self) -> #pallets_with_constants::constants::ConstantsApi {
+                            #pallets_with_constants::constants::ConstantsApi
+                        }
+                    )*
+                }
+
+                pub struct StorageApi<'a, T: ::subxt::Config> {
                     client: &'a ::subxt::Client<T>,
                 }
 
                 impl<'a, T> StorageApi<'a, T>
                 where
-                    T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>,
+                    T: ::subxt::Config,
                 {
                     #(
                         pub fn #pallets_with_storage(&self) -> #pallets_with_storage::storage::StorageApi<'a, T> {
@@ -300,16 +351,19 @@ impl RuntimeGenerator {
                     )*
                 }
 
-                pub struct TransactionApi<'a, T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>> {
+                pub struct TransactionApi<'a, T: ::subxt::Config, X, A> {
                     client: &'a ::subxt::Client<T>,
+                    marker: ::core::marker::PhantomData<(X, A)>,
                 }
 
-                impl<'a, T> TransactionApi<'a, T>
+                impl<'a, T, X, A> TransactionApi<'a, T, X, A>
                 where
-                    T: ::subxt::Config + ::subxt::ExtrinsicExtraData<T>,
+                    T: ::subxt::Config,
+                    X: ::subxt::SignedExtra<T>,
+                    A: ::subxt::AccountData,
                 {
                     #(
-                        pub fn #pallets_with_calls(&self) -> #pallets_with_calls::calls::TransactionApi<'a, T> {
+                        pub fn #pallets_with_calls(&self) -> #pallets_with_calls::calls::TransactionApi<'a, T, X, A> {
                             #pallets_with_calls::calls::TransactionApi::new(self.client)
                         }
                     )*
@@ -319,21 +373,99 @@ impl RuntimeGenerator {
     }
 }
 
-pub fn generate_structs_from_variants(
+/// Most chains require a valid account nonce as part of the extrinsic, so the default behaviour of
+/// the client is to fetch the nonce for the current account.
+///
+/// The account index (aka nonce) is commonly stored in the `System` pallet's `Account` storage item.
+/// This function attempts to find that storage item, and if it is present will implement the
+/// `subxt::AccountData` trait for it. This allows the client to construct the appropriate
+/// storage key from the account id, and then retrieve the `nonce` from the resulting storage item.
+fn generate_default_account_data_impl(
+    pallets_with_mod_names: &[(&PalletMetadata<PortableForm>, syn::Ident)],
+    default_impl_name: &syn::Ident,
     type_gen: &TypeGenerator,
+) -> Option<TokenStream2> {
+    let storage = pallets_with_mod_names
+        .iter()
+        .find(|(pallet, _)| pallet.name == "System")
+        .and_then(|(pallet, _)| pallet.storage.as_ref())?;
+    let storage_entry = storage
+        .entries
+        .iter()
+        .find(|entry| entry.name == "Account")?;
+
+    // resolve the concrete types for `AccountId` (to build the key) and `Index` to extract the
+    // account index (nonce) value from the result.
+    let (account_id_ty, account_nonce_ty) =
+        if let StorageEntryType::Map { key, value, .. } = &storage_entry.ty {
+            let account_id_ty = type_gen.resolve_type_path(key.id(), &[]);
+            let account_data_ty = type_gen.resolve_type(value.id());
+            let nonce_field = if let scale_info::TypeDef::Composite(composite) =
+                account_data_ty.type_def()
+            {
+                composite
+                    .fields()
+                    .iter()
+                    .find(|f| f.name() == Some(&"nonce".to_string()))?
+            } else {
+                abort_call_site!("Expected a `nonce` field in the account info struct")
+            };
+            let account_nonce_ty = type_gen.resolve_type_path(nonce_field.ty().id(), &[]);
+            (account_id_ty, account_nonce_ty)
+        } else {
+            abort_call_site!("System::Account should be a `StorageEntryType::Map`")
+        };
+
+    // this path to the storage entry depends on storage codegen.
+    let storage_entry_path = quote!(self::system::storage::Account);
+
+    Some(quote! {
+        /// The default storage entry from which to fetch an account nonce, required for
+        /// constructing a transaction.
+        pub enum #default_impl_name {}
+
+        impl ::subxt::AccountData for #default_impl_name {
+            type StorageEntry = #storage_entry_path;
+            type AccountId = #account_id_ty;
+            type Index = #account_nonce_ty;
+
+            fn nonce(result: &<Self::StorageEntry as ::subxt::StorageEntry>::Value) -> Self::Index {
+                result.nonce
+            }
+            fn storage_entry(account_id: Self::AccountId) -> Self::StorageEntry {
+                #storage_entry_path(account_id)
+            }
+        }
+    })
+}
+
+pub fn generate_structs_from_variants<'a, F>(
+    type_gen: &'a TypeGenerator,
     type_id: u32,
+    variant_to_struct_name: F,
     error_message_type_name: &str,
-) -> Vec<StructDef> {
+) -> Vec<CompositeDef>
+where
+    F: Fn(&str) -> std::borrow::Cow<str>,
+{
     let ty = type_gen.resolve_type(type_id);
     if let scale_info::TypeDef::Variant(variant) = ty.type_def() {
         variant
             .variants()
             .iter()
             .map(|var| {
-                StructDef::new(
-                    var.name(),
+                let struct_name = variant_to_struct_name(var.name());
+                let fields = CompositeDefFields::from_scale_info_fields(
+                    struct_name.as_ref(),
                     var.fields(),
-                    Some(syn::parse_quote!(pub)),
+                    &[],
+                    type_gen,
+                );
+                CompositeDef::struct_def(
+                    var.name(),
+                    Default::default(),
+                    fields,
+                    Some(parse_quote!(pub)),
                     type_gen,
                 )
             })
