@@ -1,4 +1,4 @@
-// Copyright 2019-2021 Parity Technologies (UK) Ltd.
+// Copyright 2019-2022 Parity Technologies (UK) Ltd.
 // This file is part of subxt.
 //
 // subxt is free software: you can redistribute it and/or modify
@@ -22,7 +22,7 @@ use scale_info::{
     TypeInfo,
 };
 
-const MOD_PATH: &'static [&'static str] = &["subxt_codegen", "types", "tests"];
+const MOD_PATH: &[&str] = &["subxt_codegen", "types", "tests"];
 
 fn get_mod<'a>(module: &'a Module, path_segs: &[&'static str]) -> Option<&'a Module<'a>> {
     let (mod_name, rest) = path_segs.split_first()?;
@@ -64,7 +64,7 @@ fn generate_struct_with_primitives() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub a: ::core::primitive::bool,
                     pub b: ::core::primitive::u32,
@@ -110,12 +110,12 @@ fn generate_struct_with_a_struct_field() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Child {
                     pub a: ::core::primitive::i32,
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Parent {
                     pub a: ::core::primitive::bool,
                     pub b: root::subxt_codegen::types::tests::Child,
@@ -155,10 +155,10 @@ fn generate_tuple_struct() {
                 pub mod tests {
                     use super::root;
 
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                     pub struct Child(pub ::core::primitive::i32,);
 
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                     pub struct Parent(pub ::core::primitive::bool, pub root::subxt_codegen::types::tests::Child,);
                 }
             }
@@ -237,44 +237,34 @@ fn derive_compact_as_for_uint_wrapper_structs() {
             pub mod tests {
                 use super::root;
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct Su128 { pub a: ::core::primitive::u128, }
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct Su16 { pub a: ::core::primitive::u16, }
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct Su32 { pub a: ::core::primitive::u32, }
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct Su64 { pub a: ::core::primitive::u64, }
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct Su8 { pub a: ::core::primitive::u8, }
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct TSu128(pub ::core::primitive::u128,);
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct TSu16(pub ::core::primitive::u16,);
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct TSu32(pub ::core::primitive::u32,);
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct TSu64(pub ::core::primitive::u64,);
 
-                #[derive(::subxt::codec::CompactAs)]
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                 pub struct TSu8(pub ::core::primitive::u8,);
             }
         }
@@ -310,7 +300,7 @@ fn generate_enum() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub enum E {
                     # [codec (index = 0)]
                     A,
@@ -325,6 +315,73 @@ fn generate_enum() {
     )
 }
 
+#[test]
+fn compact_fields() {
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct S {
+        #[codec(compact)]
+        a: u32,
+    }
+
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct TupleStruct(#[codec(compact)] u32);
+
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    enum E {
+        A {
+            #[codec(compact)]
+            a: u32,
+        },
+        B(#[codec(compact)] u32),
+    }
+
+    let mut registry = Registry::new();
+    registry.register_type(&meta_type::<S>());
+    registry.register_type(&meta_type::<TupleStruct>());
+    registry.register_type(&meta_type::<E>());
+    let portable_types: PortableRegistry = registry.into();
+
+    let type_gen = TypeGenerator::new(
+        &portable_types,
+        "root",
+        Default::default(),
+        Default::default(),
+    );
+    let types = type_gen.generate_types_mod();
+    let tests_mod = get_mod(&types, MOD_PATH).unwrap();
+
+    assert_eq!(
+        tests_mod.into_token_stream().to_string(),
+        quote! {
+            pub mod tests {
+                use super::root;
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                pub enum E {
+                    # [codec (index = 0)]
+                    A {
+                        #[codec(compact)]
+                        a: ::core::primitive::u32,
+                    },
+                    # [codec (index = 1)]
+                    B( #[codec(compact)] ::core::primitive::u32,),
+                }
+
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                pub struct S {
+                    #[codec(compact)] pub a: ::core::primitive::u32,
+                }
+
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                pub struct TupleStruct(#[codec(compact)] pub ::core::primitive::u32,);
+            }
+        }
+        .to_string()
+    )
+}
+
 #[test]
 fn generate_array_field() {
     #[allow(unused)]
@@ -351,7 +408,7 @@ fn generate_array_field() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub a: [::core::primitive::u8; 32usize],
                 }
@@ -388,7 +445,7 @@ fn option_fields() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub a: ::core::option::Option<::core::primitive::bool>,
                     pub b: ::core::option::Option<::core::primitive::u32>,
@@ -428,7 +485,7 @@ fn box_fields_struct() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub a: ::std::boxed::Box<::core::primitive::bool>,
                     pub b: ::std::boxed::Box<::core::primitive::u32>,
@@ -468,7 +525,7 @@ fn box_fields_enum() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub enum E {
                     # [codec (index = 0)]
                     A(::std::boxed::Box<::core::primitive::bool>,),
@@ -508,7 +565,7 @@ fn range_fields() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub a: ::core::ops::Range<::core::primitive::u32>,
                     pub b: ::core::ops::RangeInclusive<::core::primitive::u32>,
@@ -552,12 +609,12 @@ fn generics() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Bar {
                     pub b: root::subxt_codegen::types::tests::Foo<::core::primitive::u32>,
                     pub c: root::subxt_codegen::types::tests::Foo<::core::primitive::u8>,
                 }
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Foo<_0> {
                     pub a: _0,
                 }
@@ -600,12 +657,12 @@ fn generics_nested() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Bar<_0> {
                     pub b: root::subxt_codegen::types::tests::Foo<_0, ::core::primitive::u32>,
                 }
 
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct Foo<_0, _1> {
                     pub a: _0,
                     pub b: ::core::option::Option<(_0, _1,)>,
@@ -651,7 +708,7 @@ fn generate_bitvec() {
         quote! {
             pub mod tests {
                 use super::root;
-                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                 pub struct S {
                     pub lsb: ::subxt::bitvec::vec::BitVec<root::bitvec::order::Lsb0, ::core::primitive::u8>,
                     pub msb: ::subxt::bitvec::vec::BitVec<root::bitvec::order::Msb0, ::core::primitive::u16>,
@@ -704,16 +761,15 @@ fn generics_with_alias_adds_phantom_data_marker() {
             quote! {
                 pub mod tests {
                     use super::root;
-                    #[derive(::subxt::codec::CompactAs)]
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug, ::subxt::codec::CompactAs)]
                     pub struct NamedFields<_0> {
                         pub b: ::core::primitive::u32,
-                        #[codec(skip)] pub __subxt_unused_type_params: ::core::marker::PhantomData<_0>,
+                        #[codec(skip)] pub __subxt_unused_type_params: ::core::marker::PhantomData<_0>
                     }
-                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                    #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                     pub struct UnnamedFields<_0, _1> (
                         pub (::core::primitive::u32, ::core::primitive::u32,),
-                        #[codec(skip)] pub ::core::marker::PhantomData<(_0, _1)>,
+                        #[codec(skip)] pub ::core::marker::PhantomData<(_0, _1)>
                     );
                 }
             }
@@ -723,11 +779,11 @@ fn generics_with_alias_adds_phantom_data_marker() {
 
 #[test]
 fn modules() {
-    mod modules {
+    mod m {
         pub mod a {
             #[allow(unused)]
             #[derive(scale_info::TypeInfo)]
-            pub struct Foo {}
+            pub struct Foo;
 
             pub mod b {
                 #[allow(unused)]
@@ -748,7 +804,7 @@ fn modules() {
     }
 
     let mut registry = Registry::new();
-    registry.register_type(&meta_type::<modules::c::Foo>());
+    registry.register_type(&meta_type::<m::c::Foo>());
     let portable_types: PortableRegistry = registry.into();
 
     let type_gen = TypeGenerator::new(
@@ -765,7 +821,7 @@ fn modules() {
         quote! {
             pub mod tests {
                 use super::root;
-                pub mod modules {
+                pub mod m {
                     use super::root;
                     pub mod a {
                         use super::root;
@@ -773,22 +829,22 @@ fn modules() {
                         pub mod b {
                             use super::root;
 
-                            #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                            #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                             pub struct Bar {
-                                pub a: root::subxt_codegen::types::tests::modules::a::Foo,
+                                pub a: root::subxt_codegen::types::tests::m::a::Foo,
                             }
                         }
 
-                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
-                        pub struct Foo {}
+                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                        pub struct Foo;
                     }
 
                     pub mod c {
                         use super::root;
 
-                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode)]
+                        #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
                         pub struct Foo {
-                            pub a: root::subxt_codegen::types::tests::modules::a::b::Bar,
+                            pub a: root::subxt_codegen::types::tests::m::a::b::Bar,
                         }
                     }
                 }
@@ -797,3 +853,36 @@ fn modules() {
         .to_string()
     )
 }
+
+#[test]
+fn dont_force_struct_names_camel_case() {
+    #[allow(unused)]
+    #[derive(TypeInfo)]
+    struct AB;
+
+    let mut registry = Registry::new();
+    registry.register_type(&meta_type::<AB>());
+    let portable_types: PortableRegistry = registry.into();
+
+    let type_gen = TypeGenerator::new(
+        &portable_types,
+        "root",
+        Default::default(),
+        Default::default(),
+    );
+    let types = type_gen.generate_types_mod();
+    let tests_mod = get_mod(&types, MOD_PATH).unwrap();
+
+    assert_eq!(
+        tests_mod.into_token_stream().to_string(),
+        quote! {
+            pub mod tests {
+                use super::root;
+
+                #[derive(::subxt::codec::Encode, ::subxt::codec::Decode, Debug)]
+                pub struct AB;
+            }
+        }
+        .to_string()
+    )
+}
